@@ -1305,6 +1305,54 @@ START_TEST(quirks_model_zero)
 }
 END_TEST
 
+START_TEST(quirks_model_override)
+{
+	struct litest_device *dev = litest_current_device();
+	struct udev_device *ud = libinput_device_get_udev_device(dev->libinput_device);
+	struct quirks_context *ctx;
+	char *quirks_file;
+	struct data_dir dd;
+	struct quirks *q;
+	bool isset;
+	bool set = _i; /* ranged test */
+
+	/* Test model quirks override by setting, then unsetting (or the
+	   other way round) */
+	int rc = xasprintf(&quirks_file,
+			   "[first]\n"
+			   "MatchUdevType=mouse\n"
+			   "ModelAppleTouchpad=%d\n"
+			   "\n"
+			   "[second]\n"
+			   "MatchUdevType=mouse\n"
+			   "ModelAppleTouchpad=%d\n",
+			   set ? 0 : 1,
+			   set ? 1 : 0);
+	ck_assert_int_ne(rc, -1);
+
+	dd = make_data_dir(quirks_file);
+
+	ctx = quirks_init_subsystem(dd.dirname,
+				    NULL,
+				    log_handler,
+				    NULL,
+				    QLOG_CUSTOM_LOG_PRIORITIES);
+	ck_assert_notnull(ctx);
+
+	q = quirks_fetch_for_device(ctx, ud);
+	ck_assert_notnull(q);
+
+	ck_assert(quirks_get_bool(q, QUIRK_MODEL_APPLE_TOUCHPAD, &isset));
+	ck_assert(isset == set);
+
+	quirks_unref(q);
+	quirks_context_unref(ctx);
+	cleanup_data_dir(dd);
+	udev_device_unref(ud);
+	free(quirks_file);
+}
+END_TEST
+
 START_TEST(quirks_model_alps)
 {
 	struct litest_device *dev = litest_current_device();
@@ -1424,59 +1472,62 @@ END_TEST
 
 TEST_COLLECTION(quirks)
 {
-	litest_add_deviceless("quirks:datadir", quirks_invalid_dir);
-	litest_add_deviceless("quirks:datadir", quirks_empty_dir);
+	struct range boolean = {0, 2};
 
-	litest_add_deviceless("quirks:structure", quirks_section_empty);
-	litest_add_deviceless("quirks:structure", quirks_section_double);
-	litest_add_deviceless("quirks:structure", quirks_section_missing_match);
-	litest_add_deviceless("quirks:structure", quirks_section_missing_attr);
-	litest_add_deviceless("quirks:structure", quirks_section_match_after_attr);
-	litest_add_deviceless("quirks:structure", quirks_section_duplicate_match);
-	litest_add_deviceless("quirks:structure", quirks_section_duplicate_attr);
+	litest_add_deviceless(quirks_invalid_dir);
+	litest_add_deviceless(quirks_empty_dir);
 
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_section);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_trailing_whitespace);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_unknown_match);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_unknown_attr);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_unknown_model);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_unknown_prefix);
-	litest_add_deviceless("quirks:parsing", quirks_parse_error_model_not_one);
-	litest_add_deviceless("quirks:parsing", quirks_parse_comment_inline);
-	litest_add_deviceless("quirks:parsing", quirks_parse_comment_empty);
-	litest_add_deviceless("quirks:parsing", quirks_parse_string_quotes_single);
-	litest_add_deviceless("quirks:parsing", quirks_parse_string_quotes_double);
+	litest_add_deviceless(quirks_section_empty);
+	litest_add_deviceless(quirks_section_double);
+	litest_add_deviceless(quirks_section_missing_match);
+	litest_add_deviceless(quirks_section_missing_attr);
+	litest_add_deviceless(quirks_section_match_after_attr);
+	litest_add_deviceless(quirks_section_duplicate_match);
+	litest_add_deviceless(quirks_section_duplicate_attr);
 
-	litest_add_deviceless("quirks:parsing", quirks_parse_bustype);
-	litest_add_deviceless("quirks:parsing", quirks_parse_bustype_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_vendor);
-	litest_add_deviceless("quirks:parsing", quirks_parse_vendor_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_product);
-	litest_add_deviceless("quirks:parsing", quirks_parse_product_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_version);
-	litest_add_deviceless("quirks:parsing", quirks_parse_version_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_name);
-	litest_add_deviceless("quirks:parsing", quirks_parse_name_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_udev);
-	litest_add_deviceless("quirks:parsing", quirks_parse_udev_invalid);
-	litest_add_deviceless("quirks:parsing", quirks_parse_dmi);
-	litest_add_deviceless("quirks:parsing", quirks_parse_dmi_invalid);
+	litest_add_deviceless(quirks_parse_error_section);
+	litest_add_deviceless(quirks_parse_error_trailing_whitespace);
+	litest_add_deviceless(quirks_parse_error_unknown_match);
+	litest_add_deviceless(quirks_parse_error_unknown_attr);
+	litest_add_deviceless(quirks_parse_error_unknown_model);
+	litest_add_deviceless(quirks_parse_error_unknown_prefix);
+	litest_add_deviceless(quirks_parse_error_model_not_one);
+	litest_add_deviceless(quirks_parse_comment_inline);
+	litest_add_deviceless(quirks_parse_comment_empty);
+	litest_add_deviceless(quirks_parse_string_quotes_single);
+	litest_add_deviceless(quirks_parse_string_quotes_double);
 
-	litest_add_for_device("quirks:parsing", quirks_parse_dimension_attr, LITEST_MOUSE);
-	litest_add_for_device("quirks:parsing", quirks_parse_range_attr, LITEST_MOUSE);
-	litest_add_for_device("quirks:parsing", quirks_parse_uint_attr, LITEST_MOUSE);
-	litest_add_for_device("quirks:parsing", quirks_parse_double_attr, LITEST_MOUSE);
-	litest_add_for_device("quirks:parsing", quirks_parse_string_attr, LITEST_MOUSE);
-	litest_add_for_device("quirks:parsing", quirks_parse_integration_attr, LITEST_MOUSE);
+	litest_add_deviceless(quirks_parse_bustype);
+	litest_add_deviceless(quirks_parse_bustype_invalid);
+	litest_add_deviceless(quirks_parse_vendor);
+	litest_add_deviceless(quirks_parse_vendor_invalid);
+	litest_add_deviceless(quirks_parse_product);
+	litest_add_deviceless(quirks_parse_product_invalid);
+	litest_add_deviceless(quirks_parse_version);
+	litest_add_deviceless(quirks_parse_version_invalid);
+	litest_add_deviceless(quirks_parse_name);
+	litest_add_deviceless(quirks_parse_name_invalid);
+	litest_add_deviceless(quirks_parse_udev);
+	litest_add_deviceless(quirks_parse_udev_invalid);
+	litest_add_deviceless(quirks_parse_dmi);
+	litest_add_deviceless(quirks_parse_dmi_invalid);
 
-	litest_add_for_device("quirks:model", quirks_model_one, LITEST_MOUSE);
-	litest_add_for_device("quirks:model", quirks_model_zero, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_dimension_attr, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_range_attr, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_uint_attr, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_double_attr, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_string_attr, LITEST_MOUSE);
+	litest_add_for_device(quirks_parse_integration_attr, LITEST_MOUSE);
 
-	litest_add("quirks:devices", quirks_model_alps, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("quirks:devices", quirks_model_wacom, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("quirks:devices", quirks_model_apple, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("quirks:devices", quirks_model_synaptics_serial, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add_for_device(quirks_model_one, LITEST_MOUSE);
+	litest_add_for_device(quirks_model_zero, LITEST_MOUSE);
+	litest_add_ranged_for_device(quirks_model_override, LITEST_MOUSE, &boolean);
 
-	litest_add_deviceless("quirks:misc", quirks_call_NULL);
-	litest_add_deviceless("quirks:misc", quirks_ctx_ref);
+	litest_add(quirks_model_alps, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(quirks_model_wacom, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(quirks_model_apple, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(quirks_model_synaptics_serial, LITEST_TOUCHPAD, LITEST_ANY);
+
+	litest_add_deviceless(quirks_call_NULL);
+	litest_add_deviceless(quirks_ctx_ref);
 }
